summaryrefslogtreecommitdiffstats
path: root/net/socket/client_socket_pool_base.cc
diff options
context:
space:
mode:
authormmenke@chromium.org <mmenke@chromium.org@0039d316-1c4b-4281-b951-d872f2087c98>2013-06-11 11:06:33 +0000
committermmenke@chromium.org <mmenke@chromium.org@0039d316-1c4b-4281-b951-d872f2087c98>2013-06-11 11:06:33 +0000
commitb021ece6f16f7e67b09e65371b595feb51cdf77e (patch)
treeff06c968a970c2cc445889285790f8f795798bd8 /net/socket/client_socket_pool_base.cc
parent8f4492e9786ab5c1ee98750fb9c2479e472af476 (diff)
downloadchromium_src-b021ece6f16f7e67b09e65371b595feb51cdf77e.zip
chromium_src-b021ece6f16f7e67b09e65371b595feb51cdf77e.tar.gz
chromium_src-b021ece6f16f7e67b09e65371b595feb51cdf77e.tar.bz2
net: Socket pools prioritize requests with ignore_limits.
Also, when a socket request is cancelled when there are more ConnectJobs than socket requests for a group, no ConnectJobs are cancelled. These two changes are needed to prevent a lockup due to a socket created for sync XHR's socket being giving to another socket request of equal or higher priority. This issue may well still exist with SPDY sessions. BUG=155536 Review URL: https://chromiumcodereview.appspot.com/15927019 git-svn-id: svn://svn.chromium.org/chrome/trunk/src@205491 0039d316-1c4b-4281-b951-d872f2087c98
Diffstat (limited to 'net/socket/client_socket_pool_base.cc')
-rw-r--r--net/socket/client_socket_pool_base.cc42
1 files changed, 33 insertions, 9 deletions
diff --git a/net/socket/client_socket_pool_base.cc b/net/socket/client_socket_pool_base.cc
index 028d606..3f0ce28 100644
--- a/net/socket/client_socket_pool_base.cc
+++ b/net/socket/client_socket_pool_base.cc
@@ -19,6 +19,8 @@
using base::TimeDelta;
+namespace net {
+
namespace {
// Indicate whether we should enable idle socket cleanup timer. When timer is
@@ -37,9 +39,29 @@ const int kCleanupInterval = 10; // DO NOT INCREASE THIS TIMEOUT.
// after a certain timeout has passed without receiving an ACK.
bool g_connect_backup_jobs_enabled = true;
-} // namespace
+// Compares the effective priority of two results, and returns 1 if |request1|
+// has greater effective priority than |request2|, 0 if they have the same
+// effective priority, and -1 if |request2| has the greater effective priority.
+// Requests with |ignore_limits| set have higher effective priority than those
+// without. If both requests have |ignore_limits| set/unset, then the request
+// with the highest Pririoty has the highest effective priority. Does not take
+// into account the fact that Requests are serviced in FIFO order if they would
+// otherwise have the same priority.
+int CompareEffectiveRequestPriority(
+ const internal::ClientSocketPoolBaseHelper::Request& request1,
+ const internal::ClientSocketPoolBaseHelper::Request& request2) {
+ if (request1.ignore_limits() && !request2.ignore_limits())
+ return 1;
+ if (!request1.ignore_limits() && request2.ignore_limits())
+ return -1;
+ if (request1.priority() > request2.priority())
+ return 1;
+ if (request1.priority() < request2.priority())
+ return -1;
+ return 0;
+}
-namespace net {
+} // namespace
ConnectJob::ConnectJob(const std::string& group_name,
base::TimeDelta timeout_duration,
@@ -187,16 +209,16 @@ ClientSocketPoolBaseHelper::CallbackResultPair::CallbackResultPair(
ClientSocketPoolBaseHelper::CallbackResultPair::~CallbackResultPair() {}
-// InsertRequestIntoQueue inserts the request into the queue based on
-// priority. Highest priorities are closest to the front. Older requests are
-// prioritized over requests of equal priority.
-//
// static
void ClientSocketPoolBaseHelper::InsertRequestIntoQueue(
const Request* r, RequestQueue* pending_requests) {
RequestQueue::iterator it = pending_requests->begin();
- while (it != pending_requests->end() && r->priority() <= (*it)->priority())
+ // TODO(mmenke): Should the network stack require requests with
+ // |ignore_limits| have the highest priority?
+ while (it != pending_requests->end() &&
+ CompareEffectiveRequestPriority(*r, *(*it)) <= 0) {
++it;
+ }
pending_requests->insert(it, r);
}
@@ -494,8 +516,10 @@ void ClientSocketPoolBaseHelper::CancelRequest(
req->net_log().AddEvent(NetLog::TYPE_CANCELLED);
req->net_log().EndEvent(NetLog::TYPE_SOCKET_POOL);
- // We let the job run, unless we're at the socket limit.
- if (group->jobs().size() && ReachedMaxSocketsLimit()) {
+ // We let the job run, unless we're at the socket limit and there is
+ // not another request waiting on the job.
+ if (group->jobs().size() > group->pending_requests().size() &&
+ ReachedMaxSocketsLimit()) {
RemoveConnectJob(*group->jobs().begin(), group);
CheckForStalledSocketGroups();
}